from typing import List

from ..exceptions import PrefixNotDefinedError


class CommandOptions:
    def __init__(
        self, prefix: str = None, overruling_commands: List[str] = None
    ) -> None:

        if prefix is None:
            raise PrefixNotDefinedError("No prefix")

        if overruling_commands is None:
            overruling_commands = []

        self.prefix = prefix
        self.overruling_commands = overruling_commands



from typing import List


class RustEntityInfoItem:
    def __init__(self, data) -> None:
        self._item_id: int = data.itemId
        self._quantity: int = data.quantity
        self._item_is_blueprint: bool = data.itemIsBlueprint

    @property
    def item_id(self) -> int:
        return self._item_id

    @property
    def quantity(self) -> int:
        return self._quantity

    @property
    def item_is_blueprint(self) -> bool:
        return self._item_is_blueprint

    def __str__(self) -> str:
        return (
            "RustEntityInfoItem[item_id={}, quantity={}, item_is_blueprint={}]".format(
                self._item_id, self._quantity, self._item_is_blueprint
            )
        )


class RustEntityInfo:
    def __init__(self, data) -> None:
        self._type: int = data.type
        self._value: bool = data.payload.value
        self._items = [RustEntityInfoItem(item) for item in data.payload.items]
        self._capacity: int = data.payload.capacity
        self._has_protection: bool = data.payload.hasProtection
        self._protection_expiry: int = data.payload.protectionExpiry

    @property
    def type(self) -> int:
        return self._type

    @property
    def value(self) -> bool:
        return self._value

    @property
    def items(self) -> List[RustEntityInfoItem]:
        return self._items

    @property
    def capacity(self) -> int:
        return self._capacity

    @property
    def has_protection(self) -> bool:
        return self._has_protection

    @property
    def protection_expiry(self) -> int:
        return self._protection_expiry

    def __str__(self) -> str:
        return "RustEntityInfo[type={}, value={}, items={}, capacity={}, has_protection={}, protection_expiry={}]".format(
            self._type,
            self._value,
            self._items,
            self._capacity,
            self._has_protection,
            self._protection_expiry,
        )



class RustItem:
    def __init__(
        self, name: str, item_id: int, quantity: int, is_blueprint: bool
    ) -> None:
        self._name: str = name
        self._item_id: int = item_id
        self._quantity: int = quantity
        self._is_blueprint: bool = is_blueprint

    @property
    def name(self) -> str:
        return self._name

    @property
    def item_id(self) -> int:
        return self._item_id

    @property
    def quantity(self) -> int:
        return self._quantity

    @property
    def is_blueprint(self) -> bool:
        return self._is_blueprint

    def __str__(self) -> str:
        return "RustItem[name={}, item_id={}, quantity={}, is_blueprint={}]".format(
            self._name, self._item_id, self._quantity, self._is_blueprint
        )



import requests
import logging
import json
from datetime import datetime


class MagicValueGrabber:
    @staticmethod
    def get_magic_value() -> int:

        try:
            data = requests.get(
                "https://exp.host/@facepunch/RustCompanion",
                headers={
                    "Exponent-SDK-Version": "40.0.0",  # These are the two lines that determine the version of data that
                    "Exponent-Version": "0.0.21",  # you receive as far as I can tell from my limited debugging. As of
                    # now (8th April 2022), this is the latest
                    "Exponent-Platform": "android",
                    "Accept": "application/expo+json,application/json",
                    "Expo-Release-Channel": "default",
                    "Expo-Api-Version": "1",
                    "Expo-Client-Environment": "STANDALONE",
                    "Exponent-Accept-Signature": "true",
                    "Expo-JSON-Error": "true",
                    "Host": "exp.host",
                    "User-Agent": "okhttp/3.12.1",
                },
            )

            halves = (
                str(json.loads(data.json()["manifestString"])["publishedTime"])
                .replace("Z", "")
                .split("T")
            )

            year, month, day = halves[0].split("-")
            hour, minute, second = halves[1].split(":")

            dt = datetime(int(year), int(month), int(day), int(hour), int(minute), 0)

            return int(
                ((dt - datetime.utcfromtimestamp(0)).total_seconds() * 1000.0)
                + float(second) * 1000
            )
        except Exception:
            logging.getLogger("rustplus.py").warning(
                "[Rustplus.py] Failed to get magic value from RustPlus Server"
            )
            return 1641359159846



import math
import time

from ...exceptions.exceptions import RateLimitError


class TokenBucket:
    def __init__(
        self, current: int, maximum: int, refresh_rate, refresh_amount
    ) -> None:
        self.current = current
        self.max = maximum
        self.refresh_rate = refresh_rate
        self.refresh_amount = refresh_amount
        self.last_update = time.time()

    def can_consume(self, amount) -> bool:
        if (self.current - amount) >= 0:
            return True

        return False

    def consume(self, amount: int = 1) -> None:
        self.current -= amount

    def refresh(self) -> None:
        time_now = time.time()

        time_delta = time_now - self.last_update
        self.last_update = time_now

        self.current = min([self.current + time_delta * self.refresh_amount, self.max])


class RateLimiter:
    def __init__(
        self, current: int, maximum: int, refresh_rate, refresh_amount
    ) -> None:
        self.bucket = TokenBucket(
            current=current,
            maximum=maximum,
            refresh_rate=refresh_rate,
            refresh_amount=refresh_amount,
        )
        self.last_consumed = 0
        self.refresh_per_second = self.bucket.refresh_amount / self.bucket.refresh_rate

    def can_consume(self, amount: int = 1) -> bool:
        """
        Returns whether the user can consume the amount of tokens provided
        """
        self.bucket.refresh()
        return self.bucket.can_consume(amount)

    def consume(self, amount: int = 1) -> None:
        """
        Consumes an amount of tokens from the bucket. You should first check to see whether it is possible with can_consume
        """
        self.bucket.refresh()
        if not self.can_consume(amount):
            raise RateLimitError("Not Enough Tokens")
        self.bucket.consume(amount)

    def get_estimated_delay_time(self, target_cost: int) -> float:
        """
        Returns how long until the amount of tokens needed will be available
        """
        return (
            math.ceil(
                (((target_cost - self.bucket.current) / self.refresh_per_second) + 0.1)
                * 100
            )
            / 100
        )



from typing import List


class RustMonument:
    def __init__(self, token, x, y) -> None:
        self._token: str = token
        self._x: float = x
        self._y: float = y

    @property
    def token(self) -> str:
        return self._token

    @property
    def x(self) -> float:
        return self._x

    @property
    def y(self) -> float:
        return self._y

    def __str__(self) -> str:
        return "RustMonument[token={}, x={}, y={}]".format(
            self._token, self._x, self._y
        )


class RustMap:
    def __init__(self, data) -> None:
        self._width: int = data.width
        self._height: int = data.height
        self._jpg_image: bytes = data.jpgImage
        self._margin: int = data.oceanMargin
        self._monuments = [
            RustMonument(monument.token, monument.x, monument.y)
            for monument in data.monuments
        ]
        self._background: str = None if data.background == "" else data.background

    @property
    def width(self) -> int:
        return self._width

    @property
    def height(self) -> int:
        return self._height

    @property
    def jpg_image(self) -> bytes:
        return self._jpg_image

    @property
    def margin(self) -> int:
        return self._margin

    @property
    def monuments(self) -> List[RustMonument]:
        return self._monuments

    @property
    def background(self) -> str:
        return self._background

    def __str__(self) -> str:
        return "RustMap[width={}, height={}, jpg_image={}, margin={}, monuments={}, background={}]".format(
            self._width,
            self._height,
            len(self._jpg_image),
            self._margin,
            self._monuments,
            self._background,
        )



import asyncio
import time


class HeartBeat:
    def __init__(self, rust_api) -> None:

        self.rust_api = rust_api
        self.next_run = time.time()
        self.running = False

    async def start_beat(self) -> None:

        if self.running:
            return

        self.running = True

        asyncio.create_task(self._heart_beat())

    async def _heart_beat(self) -> None:

        while True:

            if time.time() >= self.next_run:

                await self.beat()

            else:
                await asyncio.sleep(1)

    async def beat(self) -> None:

        if self.rust_api.remote.ws is not None and self.rust_api.remote.is_open():
            await self.rust_api.send_wakeup_request()

    def reset_rhythm(self) -> None:

        self.next_run = time.time() + 240



from typing import List


class RustColour:
    def __init__(self, data) -> None:
        self._x: float = data.x
        self._y: float = data.y
        self._z: float = data.z
        self._w: float = data.w

    @property
    def x(self) -> float:
        return self._x

    @property
    def y(self) -> float:
        return self._y

    @property
    def z(self) -> float:
        return self._z

    @property
    def w(self) -> float:
        return self._w

    def __str__(self) -> str:
        return "RustColour[x={}, y={}, z={}, w={}]".format(
            self._x, self._y, self._z, self._w
        )


class RustSellOrder:
    def __init__(self, data) -> None:
        self._item_id: int = data.itemId
        self._quantity: int = data.quantity
        self._currency_id: int = data.currencyId
        self._cost_per_item: int = data.costPerItem
        self._item_is_blueprint: bool = data.itemIsBlueprint
        self._currency_is_blueprint: bool = data.currencyIsBlueprint
        self._amount_in_stock: int = data.amountInStock

    @property
    def item_id(self) -> int:
        return self._item_id

    @property
    def quantity(self) -> int:
        return self._quantity

    @property
    def currency_id(self) -> int:
        return self._currency_id

    @property
    def cost_per_item(self) -> int:
        return self._cost_per_item

    @property
    def item_is_blueprint(self) -> bool:
        return self._item_is_blueprint

    @property
    def currency_is_blueprint(self) -> bool:
        return self._currency_is_blueprint

    @property
    def amount_in_stock(self) -> int:
        return self._amount_in_stock

    def __str__(self) -> str:
        return (
            "RustSellOrder[item_id={}, quantity={}, currency_id={}, cost_per_item={}, item_is_blueprint={}, "
            "currency_is_blueprint={}]".format(
                self._item_id,
                self._quantity,
                self._currency_id,
                self._cost_per_item,
                self._item_is_blueprint,
                self._currency_is_blueprint,
            )
        )


class RustMarker:
    def __init__(self, data) -> None:
        self._id: int = data.id
        self._type: int = data.type
        self._x: float = data.x
        self._y: float = data.y
        self._steam_id: int = data.steamId
        self._rotation: float = data.rotation
        self._radius: float = data.radius
        self._colour1 = RustColour(data.color1)
        self._colour2 = RustColour(data.color2)
        self._alpha: float = data.alpha
        self._name: str = data.name
        self._sell_orders = [RustSellOrder(order) for order in data.sellOrders]

    @property
    def id(self) -> int:
        return self._id

    @property
    def type(self) -> int:
        return self._type

    @property
    def x(self) -> float:
        return self._x

    @property
    def y(self) -> float:
        return self._y

    @property
    def steam_id(self) -> int:
        return self._steam_id

    @property
    def rotation(self) -> float:
        return self._rotation

    @property
    def radius(self) -> float:
        return self._radius

    @property
    def colour1(self) -> RustColour:
        return self._colour1

    @property
    def colour2(self) -> RustColour:
        return self._colour2

    @property
    def alpha(self) -> float:
        return self._alpha

    @property
    def name(self) -> str:
        return self._name

    @property
    def sell_orders(self) -> List[RustSellOrder]:
        return self._sell_orders

    def __str__(self) -> str:
        return (
            "RustMarker[id={}, type={}, x={}, y={}, steam_id={}, rotation={}, radius={}, colour1={}, colour2={}, "
            "alpha={}, name={}, sell_orders={}]".format(
                self._id,
                self._type,
                self._x,
                self._y,
                self._steam_id,
                self._rotation,
                self._radius,
                self._colour1,
                self._colour2,
                self._alpha,
                self._name,
                self._sell_orders,
            )
        )



import functools
import inspect
import warnings


def deprecated(reason):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.
    """

    def decorator(func1):

        if inspect.isclass(func1):
            fmt1 = "Call to deprecated class {name} ({reason})."
        else:
            fmt1 = "Call to deprecated function {name} ({reason})."

        @functools.wraps(func1)
        def new_func1(*args, **kwargs):
            warnings.simplefilter("always", DeprecationWarning)
            warnings.warn(
                fmt1.format(name=func1.__name__, reason=reason),
                category=DeprecationWarning,
                stacklevel=2,
            )
            warnings.simplefilter("default", DeprecationWarning)
            return func1(*args, **kwargs)

        return new_func1

    return decorator



from .rustplus_proto import *
from .rustws import RustWebsocket
from .token_bucket import RateLimiter
from .rust_remote_interface import RustRemote
from .heartbeat import HeartBeat
from .event_handler import EventHandler



import asyncio
from asyncio.futures import Future
from typing import List, Callable, Union
from PIL import Image

from .structures import *
from .remote.rustplus_proto import *
from .remote import RustRemote, HeartBeat
from ..commands import CommandOptions
from ..commands.command_data import CommandData
from ..exceptions import *
from ..utils import RegisteredListener, deprecated
from ..conversation import ConversationFactory


class BaseRustSocket:
    def __init__(
        self,
        ip: str = None,
        port: str = None,
        steam_id: int = None,
        player_token: int = None,
        command_options: CommandOptions = None,
        raise_ratelimit_exception: bool = True,
        ratelimit_limit: int = 25,
        ratelimit_refill: int = 3,
        heartbeat: HeartBeat = None,
        use_proxy: bool = False,
    ) -> None:

        if ip is None:
            raise ValueError("Ip cannot be None")
        if port is None:
            raise ValueError("Port cannot be None")
        if steam_id is None:
            raise ValueError("SteamID cannot be None")
        if player_token is None:
            raise ValueError("PlayerToken cannot be None")

        self.ip = ip
        self.port = port
        self.steam_id = steam_id
        self.player_token = player_token
        self.seq = 1
        self.command_options = command_options
        self.raise_ratelimit_exception = raise_ratelimit_exception
        self.listener_seq = 0

        self.remote = RustRemote(
            ip=self.ip,
            port=self.port,
            command_options=command_options,
            ratelimit_limit=ratelimit_limit,
            ratelimit_refill=ratelimit_refill,
            use_proxy=use_proxy,
            api=self,
            loop=asyncio.get_event_loop_policy().get_event_loop(),
        )

        if heartbeat is None:
            raise ValueError("Heartbeat cannot be None")
        self.heartbeat = heartbeat

    async def _handle_ratelimit(self, amount=1) -> None:
        """
        Handles the ratelimit for a specific request. Will sleep if tokens are not currently available and is set to wait
        :param amount: The amount to consume
        :raises RateLimitError - If the tokens are not available and is not set to wait
        :return: None
        """
        while True:

            if self.remote.ratelimiter.can_consume(amount):
                self.remote.ratelimiter.consume(amount)
                self.heartbeat.reset_rhythm()
                return

            if self.raise_ratelimit_exception:
                raise RateLimitError("Out of tokens")

            await asyncio.sleep(
                self.remote.ratelimiter.get_estimated_delay_time(amount)
            )

    def _generate_protobuf(self) -> AppRequest:
        """
        Generates the default protobuf for a request

        :return: AppRequest - The default request object
        """
        app_request = AppRequest()
        app_request.seq = self.seq
        app_request.playerId = self.steam_id
        app_request.playerToken = self.player_token

        self.seq += 1

        return app_request

    async def connect(self, retries: int = float("inf"), delay: int = 20) -> None:
        """
        Attempts to open a connection to the rust game server specified in the constructor

        :return: None
        """
        try:
            if self.remote.ws is None:
                self.remote.connect(retries=retries, delay=delay)
                await self.send_wakeup_request()
                await self.heartbeat.start_beat()
        except ConnectionRefusedError:
            raise ServerNotResponsiveError("Cannot Connect")

    async def close_connection(self) -> None:
        """
        Disconnects from the Rust Server

        :return: None
        """
        self.remote.close()

    async def disconnect(self) -> None:
        """
        Disconnects from the Rust Server

        :return: None
        """
        await self.close_connection()

    async def send_wakeup_request(self) -> None:
        """
        Sends a request to the server to wake up broadcast responses

        :return: None
        """
        await self._handle_ratelimit()

        app_request = self._generate_protobuf()
        app_request.getTime.CopyFrom(AppEmpty())

        self.remote.ignored_responses.append(app_request.seq)

        await self.remote.send_message(app_request)

    def command(
        self,
        coro: Callable = None,
        aliases: List[str] = None,
        alais_func: Callable = None,
    ) -> Union[Callable, RegisteredListener]:
        """
        A coroutine decorator used to register a command executor

        :param alais_func: The function to test the aliases against
        :param aliases: The aliases to register the command under
        :param coro: The coroutine to call when the command is called
        :return: RegisteredListener - The listener object | Callable - The callable func for the decorator
        """

        if isinstance(coro, RegisteredListener):
            coro = coro.get_coro()

        if asyncio.iscoroutinefunction(coro):
            cmd_data = CommandData(coro, asyncio.get_event_loop(), aliases, alais_func)
            self.remote.command_handler.register_command(cmd_data)
            return RegisteredListener(coro.__name__, (cmd_data.coro, cmd_data.loop))

        def wrap_func(coro):

            if self.command_options is None:
                raise CommandsNotEnabledError("Not enabled")

            if isinstance(coro, RegisteredListener):
                coro = coro.get_coro()

            cmd_data = CommandData(coro, asyncio.get_event_loop(), aliases, alais_func)
            self.remote.command_handler.register_command(cmd_data)
            return RegisteredListener(coro.__name__, (cmd_data.coro, cmd_data.loop))

        return wrap_func

    def team_event(self, coro) -> RegisteredListener:
        """
        A Decorator to register an event listener for team changes

        :param coro: The coroutine to call when a change happens
        :return: RegisteredListener - The listener object
        """

        if isinstance(coro, RegisteredListener):
            coro = coro.get_coro()

        data = (coro, asyncio.get_event_loop())
        listener = RegisteredListener("team_changed", data)
        self.remote.event_handler.register_event(listener)
        return listener

    def chat_event(self, coro) -> RegisteredListener:
        """
        A Decorator to register an event listener for chat messages

        :param coro: The coroutine to call when a message is sent
        :return: RegisteredListener - The listener object
        """

        if isinstance(coro, RegisteredListener):
            coro = coro.get_coro()

        data = (coro, asyncio.get_event_loop())
        listener = RegisteredListener("chat_message", data)

        self.remote.event_handler.register_event(listener)

        return listener

    def entity_event(self, eid):
        """
        Decorator to register a smart device listener

        :param eid: The entity id of the entity
        :return: RegisteredListener - The listener object
        :raises SmartDeviceRegistrationError
        """

        def wrap_func(coro) -> RegisteredListener:

            if isinstance(coro, RegisteredListener):
                coro = coro.get_coro()

            async def get_entity(self, eid) -> RustEntityInfo:

                await self._handle_ratelimit()

                app_request = self._generate_protobuf()
                app_request.entityId = eid
                app_request.getEntityInfo.CopyFrom(AppEmpty())

                await self.remote.send_message(app_request)

                return await self.remote.get_response(
                    app_request.seq, app_request, False
                )

            def entity_event_callback(future_inner: Future):

                entity_info = future_inner.result()

                if entity_info.response.HasField("error"):
                    raise SmartDeviceRegistrationError("Not Found")

                self.remote.event_handler.register_event(
                    RegisteredListener(
                        eid, (coro, loop, entity_info.response.entityInfo.type)
                    )
                )

            loop = asyncio.get_event_loop()
            future = asyncio.run_coroutine_threadsafe(get_entity(self, eid), loop)
            future.add_done_callback(entity_event_callback)

            return RegisteredListener(eid, (coro))

        return wrap_func

    def protobuf_received(self, coro) -> RegisteredListener:
        """
        A Decorator to register an event listener for protobuf being received on the websocket

        :param coro: The coroutine to call when the command is called
        :return: RegisteredListener - The listener object
        """

        if isinstance(coro, RegisteredListener):
            coro = coro.get_coro()

        data = (coro, asyncio.get_event_loop())
        listener = RegisteredListener("protobuf_received", data)
        self.remote.event_handler.register_event(listener)
        return listener

    def remove_listener(self, listener) -> bool:
        """
        This will remove a listener, command or event. Takes a RegisteredListener instance

        :return: Success of removal. True = Removed. False = Not Removed
        """

        if isinstance(listener, RegisteredListener):
            return self.remote.remove_listener(listener)
        return False

    @staticmethod
    async def hang() -> None:
        """
        This Will permanently put your script into a state of 'hanging' Cannot be Undone. Only do this in scripts
        using commands

        :returns Nothing, This will never return
        """

        while True:
            await asyncio.sleep(1)

    def get_conversation_factory(self) -> ConversationFactory:
        """
        Gets the current ConversationFactory object

        :returns ConversationFactory: the factory
        """
        return self.remote.conversation_factory

    async def get_time(self) -> RustTime:
        """
        Gets the current in-game time from the server.

        :returns RustTime: The Time
        """
        raise NotImplementedError("Not Implemented")

    async def send_team_message(self, message: str) -> None:
        """
        Sends a message to the in-game team chat

        :param message: The string message to send
        """
        raise NotImplementedError("Not Implemented")

    async def get_info(self) -> RustInfo:
        """
        Gets information on the Rust Server
        :return: RustInfo - The info of the server
        """
        raise NotImplementedError("Not Implemented")

    async def get_team_chat(self) -> List[RustChatMessage]:
        """
        Gets the team chat from the server

        :return List[RustChatMessage]: The chat messages in the team chat
        """
        raise NotImplementedError("Not Implemented")

    async def get_team_info(self) -> RustTeamInfo:
        """
        Gets Information on the members of your team

        :return RustTeamInfo: The info of your team
        """
        raise NotImplementedError("Not Implemented")

    async def get_markers(self) -> List[RustMarker]:
        """
        Gets all the map markers from the server

        :return List[RustMarker]: All the markers on the map
        """
        raise NotImplementedError("Not Implemented")

    async def get_map(
        self,
        add_icons: bool = False,
        add_events: bool = False,
        add_vending_machines: bool = False,
        override_images: dict = None,
    ) -> Image:
        """
        Gets an image of the map from the server with the specified additions

        :param add_icons: To add the monument icons
        :param add_events: To add the Event icons
        :param add_vending_machines: To add the vending icons
        :param override_images: To override the images pre-supplied with RustPlus.py
        :return Image: PIL Image
        """
        raise NotImplementedError("Not Implemented")

    async def get_raw_map_data(self) -> RustMap:
        """
        Gets the raw map data from the server

        :return RustMap: The raw map of the server
        """
        raise NotImplementedError("Not Implemented")

    async def get_entity_info(self, eid: int = None) -> RustEntityInfo:
        """
        Gets entity info from the server

        :param eid: The Entities ID
        :return RustEntityInfo: The entity Info
        """
        raise NotImplementedError("Not Implemented")

    async def turn_on_smart_switch(self, eid: int = None) -> None:
        """
        Turns on a given smart switch by entity ID

        :param eid: The Entities ID
        :return None:
        """
        raise NotImplementedError("Not Implemented")

    async def turn_off_smart_switch(self, eid: int = None) -> None:
        """
        Turns off a given smart switch by entity ID

        :param eid: The Entities ID
        :return None:
        """
        raise NotImplementedError("Not Implemented")

    async def promote_to_team_leader(self, steamid: int = None) -> None:
        """
        Promotes a given user to the team leader by their 64-bit Steam ID

        :param steamid: The SteamID of the player to promote
        :return None:
        """
        raise NotImplementedError("Not Implemented")

    async def get_current_events(self) -> List[RustMarker]:
        """
        Returns all the map markers that are for events:
        Can detect:
            - Explosion
            - CH47 (Chinook)
            - Cargo Ship
            - Locked Crate
            - Attack Helicopter

        :return List[RustMarker]: All current events
        """
        raise NotImplementedError("Not Implemented")

    async def get_contents(
        self, eid: int = None, combine_stacks: bool = False
    ) -> RustContents:
        """
        Gets the contents of a storage monitor-attached container

        :param eid: The EntityID Of the storage Monitor
        :param combine_stacks: Whether to combine alike stacks together
        :return RustContents: The contents on the monitor
        """
        raise NotImplementedError("Not Implemented")

    @deprecated("Use RustSocket#get_contents")
    async def get_tc_storage_contents(
        self, eid: int = None, combine_stacks: bool = False
    ) -> RustContents:
        """
        Gets the Information about TC Upkeep and Contents.
        Do not use this for any other storage monitor than a TC
        """
        raise NotImplementedError("Not Implemented")



from typing import List


class CommandTime:
    def __init__(self, formatted_time, raw_time) -> None:
        self.formatted_time = formatted_time
        self.raw_time = raw_time


class Command:
    def __init__(
        self,
        sender_name: str,
        sender_steam_id: int,
        time: CommandTime,
        command: str,
        args: List[str],
    ) -> None:
        self.sender_name = sender_name
        self.sender_steam_id = sender_steam_id
        self.time = time
        self.command = command
        self.args = args



from typing import List

from .rust_chat_message import RustChatMessage
from .rust_team_info import RustTeamInfo


class Item:
    def __init__(self, app_message) -> None:
        self._item_id: int = app_message.itemId
        self._quantity: int = app_message.quantity
        self._item_is_blueprint: bool = app_message.itemIsBlueprint

    @property
    def item_id(self) -> int:
        return self._item_id

    @property
    def quantity(self) -> int:
        return self._quantity

    @property
    def item_is_blueprint(self) -> bool:
        return self._item_is_blueprint


class TeamEvent:
    def __init__(self, app_message) -> None:
        self._player_id: int = app_message.broadcast.teamChanged.playerId
        self._team_info = RustTeamInfo(app_message.broadcast.teamChanged.teamInfo)

    @property
    def player_id(self) -> int:
        return self._player_id

    @property
    def team_info(self) -> RustTeamInfo:
        return self._team_info


class ChatEvent:
    def __init__(self, app_message) -> None:
        self._message = RustChatMessage(app_message.broadcast.teamMessage.message)

    @property
    def message(self) -> RustChatMessage:
        return self._message


class EntityEvent:
    def __init__(self, app_message, entity_type) -> None:
        self._type = int(entity_type)
        self._entity_id: int = app_message.broadcast.entityChanged.entityId
        self._value: bool = app_message.broadcast.entityChanged.payload.value
        self._capacity: int = app_message.broadcast.entityChanged.payload.capacity
        self._has_protection: bool = (
            app_message.broadcast.entityChanged.payload.hasProtection
        )
        self._protection_expiry: int = (
            app_message.broadcast.entityChanged.payload.protectionExpiry
        )

        self._items: List[Item] = [
            Item(item) for item in app_message.broadcast.entityChanged.payload.items
        ]

    @property
    def type(self) -> int:
        return self._type

    @property
    def entity_id(self) -> int:
        return self._entity_id

    @property
    def value(self) -> bool:
        return self._value

    @property
    def capacity(self) -> int:
        return self._capacity

    @property
    def has_protection(self) -> bool:
        return self._has_protection

    @property
    def protection_expiry(self) -> int:
        return self._protection_expiry

    @property
    def items(self) -> List[Item]:
        return self._items



class ConversationPrompt:
    def __init__(self, conversation) -> None:
        self.conversation = conversation

    async def prompt(self) -> str:
        return ""

    async def on_response(self, response) -> None:
        pass

    async def on_finish(self) -> str:
        return ""



from importlib import resources
from PIL import Image
import logging
import string

from ..api.structures import RustTime


def format_time(protobuf) -> RustTime:
    def convert_time(time) -> str:
        hours, minutes = divmod(time * 60, 60)

        return (
            f"{int(hours)}:0{int(minutes)}"
            if minutes <= 9
            else f"{int(hours)}:{int(minutes)}"
        )

    sunrise = convert_time(protobuf.response.time.sunrise)
    sunset = convert_time(protobuf.response.time.sunset)
    parsed_time = convert_time(protobuf.response.time.time)

    return RustTime(
        protobuf.response.time.dayLengthMinutes,
        sunrise,
        sunset,
        parsed_time,
        protobuf.response.time.time,
    )


def format_coord(x, y, map_size) -> tuple:
    y = map_size - y - 75
    x -= 75

    if x < 0:
        x = 0
    if x > map_size:
        x = map_size - 150
    if y < 0:
        y = 0
    if y > map_size:
        y = map_size - 150

    return x, y


def convert_marker(name, angle) -> Image:
    name_to_file = {
        "2": "explosion.png",
        "4": "chinook.png",
        "5": "cargo.png",
        "6": "crate.png",
        "8": "patrol.png",
    }

    with resources.path("rustplus.api.icons", name_to_file[name]) as path:
        icon = Image.open(path).convert("RGBA")
    if name == "6":
        icon = icon.resize((85, 85))
    elif name == "2":
        icon = icon.resize((96, 96))
    elif name == "4":
        with resources.path("rustplus.api.icons", "chinook_blades.png") as path:
            blades = Image.open(path).convert("RGBA")
        blades = blades.resize((100, 100))
        icon.paste(blades, (64 - 50, 96 - 50), blades)
        icon.paste(blades, (64 - 50, 32 - 50), blades)
    elif name == "8":
        icon = icon.resize((200, 200))
        with resources.path("rustplus.api.icons", "chinook_blades.png") as path:
            blades = Image.open(path).convert("RGBA")
        blades = blades.resize((200, 200))
        icon.paste(blades, (0, 0), blades)
    icon = icon.rotate(angle)
    return icon


def convert_monument(name: str, override_images: dict) -> Image:
    name_to_file = {
        "train_tunnel_display_name": "train.png",
        "supermarket": "supermarket.png",
        "mining_outpost_display_name": "mining_outpost.png",
        "gas_station": "oxums.png",
        "fishing_village_display_name": "fishing.png",
        "large_fishing_village_display_name": "fishing.png",
        "lighthouse_display_name": "lighthouse.png",
        "excavator": "excavator.png",
        "water_treatment_plant_display_name": "water_treatment.png",
        "train_yard_display_name": "train_yard.png",
        "outpost": "outpost.png",
        "bandit_camp": "bandit.png",
        "junkyard_display_name": "junkyard.png",
        "dome_monument_name": "dome.png",
        "satellite_dish_display_name": "satellite.png",
        "power_plant_display_name": "power_plant.png",
        "military_tunnels_display_name": "military_tunnels.png",
        "airfield_display_name": "airfield.png",
        "launchsite": "launchsite.png",
        "sewer_display_name": "sewer.png",
        "oil_rig_small": "small_oil_rig.png",
        "large_oil_rig": "large_oil_rig.png",
        "underwater_lab": "underwater_lab.png",
        "AbandonedMilitaryBase": "desert_base.png",
    }

    try:
        return override_images[name]
    except KeyError:
        pass

    if name in name_to_file:
        file_name = name_to_file[name]
        with resources.path("rustplus.api.icons", file_name) as path:
            icon = Image.open(path).convert("RGBA")
    elif (
        "mining_quarry" in name
        or "harbor" in name
        or "stables" in name
        or "swamp" in name
        or "arctic_base" in name
    ):
        if "mining_quarry" in name:
            file_name = "quarry.png"
        elif "harbor" in name:
            file_name = "harbour.png"
        elif "stables" in name:
            file_name = "stable.png"
        elif "swamp" in name:
            file_name = "swamp.png"
        elif "arctic_base" in name:
            file_name = "arctic_base.png"
        with resources.path("rustplus.api.icons", file_name) as path:
            icon = Image.open(path).convert("RGBA")
    else:
        logging.getLogger("rustplus.py").info(
            f"{name} - Has no icon, report this as an issue"
        )
        with resources.path("rustplus.api.icons", "icon.png") as path:
            icon = Image.open(path).convert("RGBA")
    return icon


def entity_type_to_string(id) -> str:
    if id == 1:
        return "Switch"
    elif id == 2:
        return "Alarm"
    elif id == 3:
        return "Storage Monitor"
    else:
        raise ValueError("Not Valid type")


def convert_xy_to_grid(
    coords: tuple, map_size: float, catch_out_of_bounds: bool = True
) -> tuple:
    grid_size = 146.25
    grids = list(string.ascii_uppercase) + [
        f"A{letter}" for letter in list(string.ascii_uppercase)
    ]

    if coords[0] > map_size or coords[0] < 0 or coords[1] > map_size or coords[1] < 0:
        if catch_out_of_bounds:
            raise ValueError("Out of bounds")

    return grids[int(coords[0] // grid_size)], int((map_size - coords[1]) // grid_size)



from typing import List
from PIL import Image
from io import BytesIO
from datetime import datetime
from collections import defaultdict
from importlib import resources

from .base_rust_api import BaseRustSocket
from .structures import (
    RustInfo,
    RustMap,
    RustMarker,
    RustChatMessage,
    RustTeamInfo,
    RustEntityInfo,
    RustContents,
    RustItem,
)
from .remote.rustplus_proto import *
from .remote import HeartBeat
from ..commands import CommandOptions
from ..exceptions import *
from ..utils import (
    RustTime,
    format_time,
    format_coord,
    convert_marker,
    convert_monument,
    translate_id_to_stack,
    deprecated,
)


class RustSocket(BaseRustSocket):
    def __init__(
        self,
        ip: str = None,
        port: str = None,
        steam_id: int = None,
        player_token: int = None,
        command_options: CommandOptions = None,
        raise_ratelimit_exception: bool = True,
        ratelimit_limit: int = 25,
        ratelimit_refill: int = 3,
        use_proxy: bool = False,
    ) -> None:
        super().__init__(
            ip=ip,
            port=port,
            steam_id=steam_id,
            player_token=player_token,
            command_options=command_options,
            raise_ratelimit_exception=raise_ratelimit_exception,
            ratelimit_limit=ratelimit_limit,
            ratelimit_refill=ratelimit_refill,
            heartbeat=HeartBeat(self),
            use_proxy=use_proxy,
        )

    async def get_time(self) -> RustTime:

        await self._handle_ratelimit()

        app_request = self._generate_protobuf()
        app_request.getTime.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        response = await self.remote.get_response(app_request.seq, app_request)

        return format_time(response)

    async def send_team_message(self, message: str) -> None:

        await self._handle_ratelimit(2)

        app_send_message = AppSendMessage()
        app_send_message.message = message

        app_request = self._generate_protobuf()
        app_request.sendTeamMessage.CopyFrom(app_send_message)

        self.remote.ignored_responses.append(app_request.seq)

        await self.remote.send_message(app_request)

    async def get_info(self) -> RustInfo:

        await self._handle_ratelimit()

        app_request = self._generate_protobuf()
        app_request.getInfo.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        response = await self.remote.get_response(app_request.seq, app_request)

        return RustInfo(response.response.info)

    async def get_team_chat(self) -> List[RustChatMessage]:

        await self._handle_ratelimit()

        app_request = self._generate_protobuf()
        app_request.getTeamChat.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        messages = (
            await self.remote.get_response(app_request.seq, app_request)
        ).response.teamChat.messages

        return [RustChatMessage(message) for message in messages]

    async def get_team_info(self):

        await self._handle_ratelimit()

        app_request = self._generate_protobuf()
        app_request.getTeamInfo.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        app_message = await self.remote.get_response(app_request.seq, app_request)

        return RustTeamInfo(app_message.response.teamInfo)

    async def get_markers(self) -> List[RustMarker]:

        await self._handle_ratelimit()

        app_request = self._generate_protobuf()
        app_request.getMapMarkers.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        app_message = await self.remote.get_response(app_request.seq, app_request)

        return [
            RustMarker(marker) for marker in app_message.response.mapMarkers.markers
        ]

    async def get_raw_map_data(self) -> RustMap:

        await self._handle_ratelimit(5)

        app_request = self._generate_protobuf()
        app_request.getMap.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        app_message = await self.remote.get_response(app_request.seq, app_request)

        return RustMap(app_message.response.map)

    async def get_map(
        self,
        add_icons: bool = False,
        add_events: bool = False,
        add_vending_machines: bool = False,
        override_images: dict = None,
    ) -> Image:

        if override_images is None:
            override_images = {}

        map_size = int((await self.get_info()).size)

        await self._handle_ratelimit(
            5 + 1
            if [add_icons, add_events, add_vending_machines].count(True) >= 1
            else 0
        )

        app_request = self._generate_protobuf()
        app_request.getMap.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        app_message = await self.remote.get_response(app_request.seq, app_request)

        game_map = app_message.response.map
        monuments = list(game_map.monuments)

        try:
            image = Image.open(BytesIO(game_map.jpgImage))
        except Exception:
            raise ImageError("Invalid bytes for the image")

        image = image.crop((500, 500, game_map.height - 500, game_map.width - 500))

        game_map = image.resize((map_size, map_size), Image.ANTIALIAS)

        if add_icons or add_events or add_vending_machines:
            map_markers = await self.get_markers()

            if add_icons:
                for monument in monuments:
                    if str(monument.token) == "DungeonBase":
                        continue
                    icon = convert_monument(monument.token, override_images)
                    if monument.token in override_images:
                        icon = icon.resize((150, 150))
                    if str(monument.token) == "train_tunnel_display_name":
                        icon = icon.resize((100, 125))
                    game_map.paste(
                        icon,
                        (format_coord(int(monument.x), int(monument.y), map_size)),
                        icon,
                    )

            if add_vending_machines:
                with resources.path(
                    "rustplus.api.icons", "vending_machine.png"
                ) as path:
                    vending_machine = Image.open(path).convert("RGBA")
                    vending_machine = vending_machine.resize((100, 100))

            for marker in map_markers:
                if add_events:
                    if (
                        marker.type == 2
                        or marker.type == 4
                        or marker.type == 5
                        or marker.type == 6
                        or marker.type == 8
                    ):
                        icon = convert_marker(str(marker.type), marker.rotation)
                        if marker.type == 6:
                            x = marker.x
                            y = marker.y
                            if y > map_size:
                                y = map_size
                            if y < 0:
                                y = 100
                            if x > map_size:
                                x = map_size - 75
                            if x < 0:
                                x = 50
                            game_map.paste(icon, (int(x), map_size - int(y)), icon)
                        else:
                            game_map.paste(
                                icon,
                                (format_coord(int(marker.x), int(marker.y), map_size)),
                                icon,
                            )
                if add_vending_machines and marker.type == 3:
                    game_map.paste(
                        vending_machine,
                        (int(marker.x) - 50, map_size - int(marker.y) - 50),
                        vending_machine,
                    )

        return game_map.resize((2000, 2000), Image.ANTIALIAS)

    async def get_entity_info(self, eid: int = None) -> RustEntityInfo:

        await self._handle_ratelimit()

        if eid is None:
            raise ValueError("EID cannot be None")

        app_request = self._generate_protobuf()
        app_request.entityId = eid
        app_request.getEntityInfo.CopyFrom(AppEmpty())

        await self.remote.send_message(app_request)

        app_message = await self.remote.get_response(app_request.seq, app_request)

        return RustEntityInfo(app_message.response.entityInfo)

    async def _update_smart_device(self, eid: int, value: bool) -> None:

        await self._handle_ratelimit()

        entity_value = AppSetEntityValue()
        entity_value.value = value

        app_request = self._generate_protobuf()

        app_request.entityId = eid
        app_request.setEntityValue.CopyFrom(entity_value)

        self.remote.ignored_responses.append(app_request.seq)

        await self.remote.send_message(app_request)

    async def turn_on_smart_switch(self, eid: int = None) -> None:

        if eid is None:
            raise ValueError("EID cannot be None")

        await self._update_smart_device(eid, True)

    async def turn_off_smart_switch(self, eid: int = None) -> None:

        if eid is None:
            raise ValueError("EID cannot be None")

        await self._update_smart_device(eid, False)

    async def promote_to_team_leader(self, steam_id: int = None) -> None:

        if steam_id is None:
            raise ValueError("SteamID cannot be None")

        await self._handle_ratelimit()

        leader_packet = AppPromoteToLeader()
        leader_packet.steamId = steam_id

        app_request = self._generate_protobuf()
        app_request.promoteToLeader.CopyFrom(leader_packet)

        self.remote.ignored_responses.append(app_request.seq)

        await self.remote.send_message(app_request)

    async def get_current_events(self) -> List[RustMarker]:

        return [
            marker
            for marker in (await self.get_markers())
            if marker.type == 2
            or marker.type == 4
            or marker.type == 5
            or marker.type == 6
            or marker.type == 8
        ]

    async def get_contents(
        self, eid: int = None, combine_stacks: bool = False
    ) -> RustContents:

        if eid is None:
            raise ValueError("EID cannot be None")

        returned_data = await self.get_entity_info(eid)

        target_time = datetime.utcfromtimestamp(int(returned_data.protection_expiry))
        difference = target_time - datetime.utcnow()

        items = []

        for item in returned_data.items:
            items.append(
                RustItem(
                    translate_id_to_stack(item.item_id),
                    item.item_id,
                    item.quantity,
                    item.item_is_blueprint,
                )
            )

        if combine_stacks:
            merged_map = defaultdict(tuple)

            for item in items:
                data = merged_map[str(item.item_id)]
                if data:
                    count = int(data[0]) + int(item.quantity)
                    merged_map[str(item.item_id)] = (count, bool(item.is_blueprint))
                else:
                    merged_map[str(item.item_id)] = (
                        int(item.quantity),
                        bool(item.is_blueprint),
                    )

            items = []
            for key in merged_map.keys():
                items.append(
                    RustItem(
                        translate_id_to_stack(key),
                        key,
                        int(merged_map[key][0]),
                        bool(merged_map[key][1]),
                    )
                )

        return RustContents(difference, bool(returned_data.has_protection), items)

    @deprecated("Use RustSocket.get_contents")
    async def get_tc_storage_contents(
        self, eid: int = None, combine_stacks: bool = False
    ) -> RustContents:
        return await self.get_contents(eid=eid, combine_stacks=combine_stacks)



from push_receiver import listen
from threading import Thread


class FCMListener:
    def __init__(self, data: dict = None):
        self.thread = None
        self.data = data

    def on_notification(self, obj, notification, data_message):
        pass

    def start(self, daemon=True):
        self.thread = Thread(target=self.__fcm_listen, daemon=daemon).start()

    def __fcm_listen(self):

        if self.data is None:
            raise ValueError("Data is None")

        listen(credentials=self.data["fcm_credentials"], callback=self.on_notification)



import asyncio
from asyncio.futures import Future
from typing import List

from ..structures import EntityEvent, TeamEvent, ChatEvent
from ...utils import RegisteredListener


class EventHandler:
    def __init__(self) -> None:
        pass

    def register_event(self, listener: RegisteredListener) -> None:

        if not asyncio.iscoroutinefunction(listener.data[0]):
            raise TypeError("The event registered must be a coroutine")

        if hasattr(self, str(listener.listener_id)):
            events: List[RegisteredListener] = getattr(self, str(listener.listener_id))
            events.append(listener)
            setattr(self, str(listener.listener_id), events)
        else:
            setattr(self, str(listener.listener_id), [listener])

    @staticmethod
    def _schedule_event(loop, coro, arg) -> None:
        def callback(inner_future: Future):
            inner_future.result()

        future: Future = asyncio.run_coroutine_threadsafe(coro(arg), loop)
        future.add_done_callback(callback)

    def run_entity_event(self, name, app_message) -> None:

        if hasattr(self, str(name)):
            for event in getattr(self, str(name)):
                coro, loop, event_type = event.data

                self._schedule_event(loop, coro, EntityEvent(app_message, event_type))

    def run_team_event(self, app_message) -> None:

        if hasattr(self, "team_changed"):
            for event in getattr(self, "team_changed"):
                coro, loop = event.data

                self._schedule_event(loop, coro, TeamEvent(app_message))

    def run_chat_event(self, app_message) -> None:

        if hasattr(self, "chat_message"):
            for event in getattr(self, "chat_message"):
                coro, loop = event.data

                self._schedule_event(loop, coro, ChatEvent(app_message))

    def run_proto_event(self, byte_data: bytes) -> None:

        if hasattr(self, "protobuf_received"):
            for event in getattr(self, "protobuf_received"):
                coro, loop = event.data

                self._schedule_event(loop, coro, byte_data)

    def has_event(self, listener: RegisteredListener) -> bool:
        if hasattr(self, listener.listener_id):
            return listener in list(getattr(self, listener.listener_id))
        return False

    def remove_event(self, listener: RegisteredListener) -> None:
        if hasattr(self, listener.listener_id):
            events = list(getattr(self, listener.listener_id))
            try:
                while True:
                    events.remove(listener)
            except ValueError:
                pass
            setattr(self, listener.listener_id, events)



r"""
RustPlus, An API wrapper for interfacing with the Rust+ App API
"""

from .api import RustSocket
from .api.structures import EntityEvent, TeamEvent, ChatEvent
from .api.remote.fcm_listener import FCMListener
from .commands import CommandOptions, Command
from .exceptions import *
from .conversation import ConversationFactory, Conversation, ConversationPrompt
from .utils import entity_type_to_string, convert_xy_to_grid

__name__ = "rustplus"
__author__ = "olijeffers0n"
__version__ = "5.3.18"
__support__ = "Discord: https://discord.gg/nQqJe8qvP8"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rustplus.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0erustplus.proto\x12\x08rustplus"`\n\nAppMessage\x12\'\n\x08response\x18\x01 \x01(\x0b\x32\x15.rustplus.AppResponse\x12)\n\tbroadcast\x18\x02 \x01(\x0b\x32\x16.rustplus.AppBroadcast"\x9b\x05\n\nAppRequest\x12\x0b\n\x03seq\x18\x01 \x02(\r\x12\x10\n\x08playerId\x18\x02 \x02(\x04\x12\x13\n\x0bplayerToken\x18\x03 \x02(\x05\x12\x10\n\x08\x65ntityId\x18\x04 \x01(\r\x12#\n\x07getInfo\x18\x08 \x01(\x0b\x32\x12.rustplus.AppEmpty\x12#\n\x07getTime\x18\t \x01(\x0b\x32\x12.rustplus.AppEmpty\x12"\n\x06getMap\x18\n \x01(\x0b\x32\x12.rustplus.AppEmpty\x12\'\n\x0bgetTeamInfo\x18\x0b \x01(\x0b\x32\x12.rustplus.AppEmpty\x12\'\n\x0bgetTeamChat\x18\x0c \x01(\x0b\x32\x12.rustplus.AppEmpty\x12\x31\n\x0fsendTeamMessage\x18\r \x01(\x0b\x32\x18.rustplus.AppSendMessage\x12)\n\rgetEntityInfo\x18\x0e \x01(\x0b\x32\x12.rustplus.AppEmpty\x12\x33\n\x0esetEntityValue\x18\x0f \x01(\x0b\x32\x1b.rustplus.AppSetEntityValue\x12-\n\x11\x63heckSubscription\x18\x10 \x01(\x0b\x32\x12.rustplus.AppEmpty\x12*\n\x0fsetSubscription\x18\x11 \x01(\x0b\x32\x11.rustplus.AppFlag\x12)\n\rgetMapMarkers\x18\x12 \x01(\x0b\x32\x12.rustplus.AppEmpty\x12\x37\n\x0egetCameraFrame\x18\x13 \x01(\x0b\x32\x1f.rustplus.AppCameraFrameRequest\x12\x35\n\x0fpromoteToLeader\x18\x14 \x01(\x0b\x32\x1c.rustplus.AppPromoteToLeader"!\n\x0e\x41ppSendMessage\x12\x0f\n\x07message\x18\x01 \x02(\t""\n\x11\x41ppSetEntityValue\x12\r\n\x05value\x18\x01 \x02(\x08":\n\x15\x41ppCameraFrameRequest\x12\x12\n\nidentifier\x18\x01 \x02(\t\x12\r\n\x05\x66rame\x18\x02 \x02(\r"%\n\x12\x41ppPromoteToLeader\x12\x0f\n\x07steamId\x18\x01 \x02(\x04"\xc1\x03\n\x0b\x41ppResponse\x12\x0b\n\x03seq\x18\x01 \x02(\r\x12%\n\x07success\x18\x04 \x01(\x0b\x32\x14.rustplus.AppSuccess\x12!\n\x05\x65rror\x18\x05 \x01(\x0b\x32\x12.rustplus.AppError\x12\x1f\n\x04info\x18\x06 \x01(\x0b\x32\x11.rustplus.AppInfo\x12\x1f\n\x04time\x18\x07 \x01(\x0b\x32\x11.rustplus.AppTime\x12\x1d\n\x03map\x18\x08 \x01(\x0b\x32\x10.rustplus.AppMap\x12\'\n\x08teamInfo\x18\t \x01(\x0b\x32\x15.rustplus.AppTeamInfo\x12\'\n\x08teamChat\x18\n \x01(\x0b\x32\x15.rustplus.AppTeamChat\x12+\n\nentityInfo\x18\x0b \x01(\x0b\x32\x17.rustplus.AppEntityInfo\x12\x1f\n\x04\x66lag\x18\x0c \x01(\x0b\x32\x11.rustplus.AppFlag\x12+\n\nmapMarkers\x18\r \x01(\x0b\x32\x17.rustplus.AppMapMarkers\x12-\n\x0b\x63\x61meraFrame\x18\x0e \x01(\x0b\x32\x18.rustplus.AppCameraFrame"\x9f\x01\n\x0c\x41ppBroadcast\x12-\n\x0bteamChanged\x18\x04 \x01(\x0b\x32\x18.rustplus.AppTeamChanged\x12-\n\x0bteamMessage\x18\x05 \x01(\x0b\x32\x18.rustplus.AppTeamMessage\x12\x31\n\rentityChanged\x18\x06 \x01(\x0b\x32\x1a.rustplus.AppEntityChanged"\n\n\x08\x41ppEmpty"\x0c\n\nAppSuccess"\x19\n\x08\x41ppError\x12\r\n\x05\x65rror\x18\x01 \x02(\t"\xc1\x01\n\x07\x41ppInfo\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0bheaderImage\x18\x02 \x02(\t\x12\x0b\n\x03url\x18\x03 \x02(\t\x12\x0b\n\x03map\x18\x04 \x02(\t\x12\x0f\n\x07mapSize\x18\x05 \x02(\r\x12\x10\n\x08wipeTime\x18\x06 \x02(\r\x12\x0f\n\x07players\x18\x07 \x02(\r\x12\x12\n\nmaxPlayers\x18\x08 \x02(\r\x12\x15\n\rqueuedPlayers\x18\t \x02(\r\x12\x0c\n\x04seed\x18\n \x01(\r\x12\x0c\n\x04salt\x18\x0b \x01(\r"e\n\x07\x41ppTime\x12\x18\n\x10\x64\x61yLengthMinutes\x18\x01 \x02(\x02\x12\x11\n\ttimeScale\x18\x02 \x02(\x02\x12\x0f\n\x07sunrise\x18\x03 \x02(\x02\x12\x0e\n\x06sunset\x18\x04 \x02(\x02\x12\x0c\n\x04time\x18\x05 \x02(\x02"\xc1\x01\n\x06\x41ppMap\x12\r\n\x05width\x18\x01 \x02(\r\x12\x0e\n\x06height\x18\x02 \x02(\r\x12\x10\n\x08jpgImage\x18\x03 \x02(\x0c\x12\x13\n\x0boceanMargin\x18\x04 \x02(\x05\x12,\n\tmonuments\x18\x05 \x03(\x0b\x32\x19.rustplus.AppMap.Monument\x12\x12\n\nbackground\x18\x06 \x01(\t\x1a/\n\x08Monument\x12\r\n\x05token\x18\x01 \x02(\t\x12\t\n\x01x\x18\x02 \x02(\x02\x12\t\n\x01y\x18\x03 \x02(\x02"\xea\x02\n\x0b\x41ppTeamInfo\x12\x15\n\rleaderSteamId\x18\x01 \x02(\x04\x12-\n\x07members\x18\x02 \x03(\x0b\x32\x1c.rustplus.AppTeamInfo.Member\x12,\n\x08mapNotes\x18\x03 \x03(\x0b\x32\x1a.rustplus.AppTeamInfo.Note\x12\x32\n\x0eleaderMapNotes\x18\x04 \x03(\x0b\x32\x1a.rustplus.AppTeamInfo.Note\x1a\x86\x01\n\x06Member\x12\x0f\n\x07steamId\x18\x01 \x02(\x04\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\t\n\x01x\x18\x03 \x02(\x02\x12\t\n\x01y\x18\x04 \x02(\x02\x12\x10\n\x08isOnline\x18\x05 \x02(\x08\x12\x11\n\tspawnTime\x18\x06 \x02(\r\x12\x0f\n\x07isAlive\x18\x07 \x02(\x08\x12\x11\n\tdeathTime\x18\x08 \x02(\r\x1a*\n\x04Note\x12\x0c\n\x04type\x18\x02 \x02(\x05\x12\t\n\x01x\x18\x03 \x02(\x02\x12\t\n\x01y\x18\x04 \x02(\x02"9\n\x0b\x41ppTeamChat\x12*\n\x08messages\x18\x01 \x03(\x0b\x32\x18.rustplus.AppChatMessage"]\n\x0e\x41ppChatMessage\x12\x0f\n\x07steamId\x18\x01 \x02(\x04\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\x0f\n\x07message\x18\x03 \x02(\t\x12\r\n\x05\x63olor\x18\x04 \x02(\t\x12\x0c\n\x04time\x18\x05 \x02(\r"c\n\rAppEntityInfo\x12%\n\x04type\x18\x01 \x02(\x0e\x32\x17.rustplus.AppEntityType\x12+\n\x07payload\x18\x03 \x02(\x0b\x32\x1a.rustplus.AppEntityPayload"\xd7\x01\n\x10\x41ppEntityPayload\x12\r\n\x05value\x18\x01 \x01(\x08\x12.\n\x05items\x18\x02 \x03(\x0b\x32\x1f.rustplus.AppEntityPayload.Item\x12\x10\n\x08\x63\x61pacity\x18\x03 \x01(\x05\x12\x15\n\rhasProtection\x18\x04 \x01(\x08\x12\x18\n\x10protectionExpiry\x18\x05 \x01(\r\x1a\x41\n\x04Item\x12\x0e\n\x06itemId\x18\x01 \x02(\x05\x12\x10\n\x08quantity\x18\x02 \x02(\x05\x12\x17\n\x0fitemIsBlueprint\x18\x03 \x02(\x08"\x18\n\x07\x41ppFlag\x12\r\n\x05value\x18\x01 \x02(\x08"5\n\rAppMapMarkers\x12$\n\x07markers\x18\x01 \x03(\x0b\x32\x13.rustplus.AppMarker"5\n\x07Vector4\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\x12\t\n\x01w\x18\x04 \x01(\x02"\xc3\x03\n\tAppMarker\x12\n\n\x02id\x18\x01 \x02(\r\x12%\n\x04type\x18\x02 \x02(\x0e\x32\x17.rustplus.AppMarkerType\x12\t\n\x01x\x18\x03 \x02(\x02\x12\t\n\x01y\x18\x04 \x02(\x02\x12\x0f\n\x07steamId\x18\x05 \x01(\x04\x12\x10\n\x08rotation\x18\x06 \x01(\x02\x12\x0e\n\x06radius\x18\x07 \x01(\x02\x12!\n\x06\x63olor1\x18\x08 \x01(\x0b\x32\x11.rustplus.Vector4\x12!\n\x06\x63olor2\x18\t \x01(\x0b\x32\x11.rustplus.Vector4\x12\r\n\x05\x61lpha\x18\n \x01(\x02\x12\x0c\n\x04name\x18\x0b \x01(\t\x12\x31\n\nsellOrders\x18\r \x03(\x0b\x32\x1d.rustplus.AppMarker.SellOrder\x1a\xa3\x01\n\tSellOrder\x12\x0e\n\x06itemId\x18\x01 \x02(\x05\x12\x10\n\x08quantity\x18\x02 \x02(\x05\x12\x12\n\ncurrencyId\x18\x03 \x02(\x05\x12\x13\n\x0b\x63ostPerItem\x18\x04 \x02(\x05\x12\x15\n\ramountInStock\x18\x05 \x02(\x05\x12\x17\n\x0fitemIsBlueprint\x18\x06 \x02(\x08\x12\x1b\n\x13\x63urrencyIsBlueprint\x18\x07 \x02(\x08"1\n\x0e\x41ppCameraFrame\x12\r\n\x05\x66rame\x18\x01 \x02(\r\x12\x10\n\x08jpgImage\x18\x02 \x02(\x0c"K\n\x0e\x41ppTeamChanged\x12\x10\n\x08playerId\x18\x01 \x02(\x04\x12\'\n\x08teamInfo\x18\x02 \x02(\x0b\x32\x15.rustplus.AppTeamInfo";\n\x0e\x41ppTeamMessage\x12)\n\x07message\x18\x01 \x02(\x0b\x32\x18.rustplus.AppChatMessage"Q\n\x10\x41ppEntityChanged\x12\x10\n\x08\x65ntityId\x18\x01 \x02(\r\x12+\n\x07payload\x18\x02 \x02(\x0b\x32\x1a.rustplus.AppEntityPayload*:\n\rAppEntityType\x12\n\n\x06Switch\x10\x01\x12\t\n\x05\x41larm\x10\x02\x12\x12\n\x0eStorageMonitor\x10\x03*\x8b\x01\n\rAppMarkerType\x12\n\n\x06Player\x10\x01\x12\r\n\tExplosion\x10\x02\x12\x12\n\x0eVendingMachine\x10\x03\x12\x08\n\x04\x43H47\x10\x04\x12\r\n\tCargoShip\x10\x05\x12\t\n\x05\x43rate\x10\x06\x12\x11\n\rGenericRadius\x10\x07\x12\x14\n\x10PatrolHelicopter\x10\x08'
)

_APPENTITYTYPE = DESCRIPTOR.enum_types_by_name["AppEntityType"]
AppEntityType = enum_type_wrapper.EnumTypeWrapper(_APPENTITYTYPE)
_APPMARKERTYPE = DESCRIPTOR.enum_types_by_name["AppMarkerType"]
AppMarkerType = enum_type_wrapper.EnumTypeWrapper(_APPMARKERTYPE)
Switch = 1
Alarm = 2
StorageMonitor = 3
Player = 1
Explosion = 2
VendingMachine = 3
CH47 = 4
CargoShip = 5
Crate = 6
GenericRadius = 7
PatrolHelicopter = 8

_APPMESSAGE = DESCRIPTOR.message_types_by_name["AppMessage"]
_APPREQUEST = DESCRIPTOR.message_types_by_name["AppRequest"]
_APPSENDMESSAGE = DESCRIPTOR.message_types_by_name["AppSendMessage"]
_APPSETENTITYVALUE = DESCRIPTOR.message_types_by_name["AppSetEntityValue"]
_APPCAMERAFRAMEREQUEST = DESCRIPTOR.message_types_by_name["AppCameraFrameRequest"]
_APPPROMOTETOLEADER = DESCRIPTOR.message_types_by_name["AppPromoteToLeader"]
_APPRESPONSE = DESCRIPTOR.message_types_by_name["AppResponse"]
_APPBROADCAST = DESCRIPTOR.message_types_by_name["AppBroadcast"]
_APPEMPTY = DESCRIPTOR.message_types_by_name["AppEmpty"]
_APPSUCCESS = DESCRIPTOR.message_types_by_name["AppSuccess"]
_APPERROR = DESCRIPTOR.message_types_by_name["AppError"]
_APPINFO = DESCRIPTOR.message_types_by_name["AppInfo"]
_APPTIME = DESCRIPTOR.message_types_by_name["AppTime"]
_APPMAP = DESCRIPTOR.message_types_by_name["AppMap"]
_APPMAP_MONUMENT = _APPMAP.nested_types_by_name["Monument"]
_APPTEAMINFO = DESCRIPTOR.message_types_by_name["AppTeamInfo"]
_APPTEAMINFO_MEMBER = _APPTEAMINFO.nested_types_by_name["Member"]
_APPTEAMINFO_NOTE = _APPTEAMINFO.nested_types_by_name["Note"]
_APPTEAMCHAT = DESCRIPTOR.message_types_by_name["AppTeamChat"]
_APPCHATMESSAGE = DESCRIPTOR.message_types_by_name["AppChatMessage"]
_APPENTITYINFO = DESCRIPTOR.message_types_by_name["AppEntityInfo"]
_APPENTITYPAYLOAD = DESCRIPTOR.message_types_by_name["AppEntityPayload"]
_APPENTITYPAYLOAD_ITEM = _APPENTITYPAYLOAD.nested_types_by_name["Item"]
_APPFLAG = DESCRIPTOR.message_types_by_name["AppFlag"]
_APPMAPMARKERS = DESCRIPTOR.message_types_by_name["AppMapMarkers"]
_VECTOR4 = DESCRIPTOR.message_types_by_name["Vector4"]
_APPMARKER = DESCRIPTOR.message_types_by_name["AppMarker"]
_APPMARKER_SELLORDER = _APPMARKER.nested_types_by_name["SellOrder"]
_APPCAMERAFRAME = DESCRIPTOR.message_types_by_name["AppCameraFrame"]
_APPTEAMCHANGED = DESCRIPTOR.message_types_by_name["AppTeamChanged"]
_APPTEAMMESSAGE = DESCRIPTOR.message_types_by_name["AppTeamMessage"]
_APPENTITYCHANGED = DESCRIPTOR.message_types_by_name["AppEntityChanged"]
AppMessage = _reflection.GeneratedProtocolMessageType(
    "AppMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPMESSAGE,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppMessage)
    },
)
_sym_db.RegisterMessage(AppMessage)

AppRequest = _reflection.GeneratedProtocolMessageType(
    "AppRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPREQUEST,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppRequest)
    },
)
_sym_db.RegisterMessage(AppRequest)

AppSendMessage = _reflection.GeneratedProtocolMessageType(
    "AppSendMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPSENDMESSAGE,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppSendMessage)
    },
)
_sym_db.RegisterMessage(AppSendMessage)

AppSetEntityValue = _reflection.GeneratedProtocolMessageType(
    "AppSetEntityValue",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPSETENTITYVALUE,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppSetEntityValue)
    },
)
_sym_db.RegisterMessage(AppSetEntityValue)

AppCameraFrameRequest = _reflection.GeneratedProtocolMessageType(
    "AppCameraFrameRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPCAMERAFRAMEREQUEST,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppCameraFrameRequest)
    },
)
_sym_db.RegisterMessage(AppCameraFrameRequest)

AppPromoteToLeader = _reflection.GeneratedProtocolMessageType(
    "AppPromoteToLeader",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPPROMOTETOLEADER,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppPromoteToLeader)
    },
)
_sym_db.RegisterMessage(AppPromoteToLeader)

AppResponse = _reflection.GeneratedProtocolMessageType(
    "AppResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPRESPONSE,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppResponse)
    },
)
_sym_db.RegisterMessage(AppResponse)

AppBroadcast = _reflection.GeneratedProtocolMessageType(
    "AppBroadcast",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPBROADCAST,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppBroadcast)
    },
)
_sym_db.RegisterMessage(AppBroadcast)

AppEmpty = _reflection.GeneratedProtocolMessageType(
    "AppEmpty",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPEMPTY,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppEmpty)
    },
)
_sym_db.RegisterMessage(AppEmpty)

AppSuccess = _reflection.GeneratedProtocolMessageType(
    "AppSuccess",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPSUCCESS,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppSuccess)
    },
)
_sym_db.RegisterMessage(AppSuccess)

AppError = _reflection.GeneratedProtocolMessageType(
    "AppError",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPERROR,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppError)
    },
)
_sym_db.RegisterMessage(AppError)

AppInfo = _reflection.GeneratedProtocolMessageType(
    "AppInfo",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPINFO,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppInfo)
    },
)
_sym_db.RegisterMessage(AppInfo)

AppTime = _reflection.GeneratedProtocolMessageType(
    "AppTime",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPTIME,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppTime)
    },
)
_sym_db.RegisterMessage(AppTime)

AppMap = _reflection.GeneratedProtocolMessageType(
    "AppMap",
    (_message.Message,),
    {
        "Monument": _reflection.GeneratedProtocolMessageType(
            "Monument",
            (_message.Message,),
            {
                "DESCRIPTOR": _APPMAP_MONUMENT,
                "__module__": "rustplus_pb2"
                # @@protoc_insertion_point(class_scope:rustplus.AppMap.Monument)
            },
        ),
        "DESCRIPTOR": _APPMAP,
        "__module__": "rustplus_pb2"
        # @@protoc_insertion_point(class_scope:rustplus.AppMap)
    },
)
_sym_db.RegisterMessage(AppMap)
_sym_db.RegisterMessage(AppMap.Monument)

AppTeamInfo = _reflection.GeneratedProtocolMessageType(
    "AppTeamInfo",
    (_message.Message,),
    {
        "Member": _reflection.GeneratedProtocolMessageType(
            "Member",
            (_message.Message,),
            {
                "DESCRIPTOR": _APPTEAMINFO_MEMBER,
                "__module__": "rustplus_pb2"
                # @@protoc_insertion_point(class_scope:rustplus.AppTeamInfo.Member)
            },
        ),
        "Note": _reflection.GeneratedProtocolMessageType(
            "Note",
            (_message.Message,),
            {
                "DESCRIPTOR": _APPTEAMINFO_NOTE,
                "__module__": "rustplus_pb2"
                # @@protoc_insertion_point(class_scope:rustplus.AppTeamInfo.Note)
            },
        ),
        "DESCRIPTOR": _APPTEAMINFO,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppTeamInfo)
    },
)
_sym_db.RegisterMessage(AppTeamInfo)
_sym_db.RegisterMessage(AppTeamInfo.Member)
_sym_db.RegisterMessage(AppTeamInfo.Note)

AppTeamChat = _reflection.GeneratedProtocolMessageType(
    "AppTeamChat",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPTEAMCHAT,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppTeamChat)
    },
)
_sym_db.RegisterMessage(AppTeamChat)

AppChatMessage = _reflection.GeneratedProtocolMessageType(
    "AppChatMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPCHATMESSAGE,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppChatMessage)
    },
)
_sym_db.RegisterMessage(AppChatMessage)

AppEntityInfo = _reflection.GeneratedProtocolMessageType(
    "AppEntityInfo",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPENTITYINFO,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppEntityInfo)
    },
)
_sym_db.RegisterMessage(AppEntityInfo)

AppEntityPayload = _reflection.GeneratedProtocolMessageType(
    "AppEntityPayload",
    (_message.Message,),
    {
        "Item": _reflection.GeneratedProtocolMessageType(
            "Item",
            (_message.Message,),
            {
                "DESCRIPTOR": _APPENTITYPAYLOAD_ITEM,
                "__module__": "rustplus_proto"
                # @@protoc_insertion_point(class_scope:rustplus.AppEntityPayload.Item)
            },
        ),
        "DESCRIPTOR": _APPENTITYPAYLOAD,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppEntityPayload)
    },
)
_sym_db.RegisterMessage(AppEntityPayload)
_sym_db.RegisterMessage(AppEntityPayload.Item)

AppFlag = _reflection.GeneratedProtocolMessageType(
    "AppFlag",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPFLAG,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppFlag)
    },
)
_sym_db.RegisterMessage(AppFlag)

AppMapMarkers = _reflection.GeneratedProtocolMessageType(
    "AppMapMarkers",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPMAPMARKERS,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppMapMarkers)
    },
)
_sym_db.RegisterMessage(AppMapMarkers)

Vector4 = _reflection.GeneratedProtocolMessageType(
    "Vector4",
    (_message.Message,),
    {
        "DESCRIPTOR": _VECTOR4,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.Vector4)
    },
)
_sym_db.RegisterMessage(Vector4)

AppMarker = _reflection.GeneratedProtocolMessageType(
    "AppMarker",
    (_message.Message,),
    {
        "SellOrder": _reflection.GeneratedProtocolMessageType(
            "SellOrder",
            (_message.Message,),
            {
                "DESCRIPTOR": _APPMARKER_SELLORDER,
                "__module__": "rustplus_proto"
                # @@protoc_insertion_point(class_scope:rustplus.AppMarker.SellOrder)
            },
        ),
        "DESCRIPTOR": _APPMARKER,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppMarker)
    },
)
_sym_db.RegisterMessage(AppMarker)
_sym_db.RegisterMessage(AppMarker.SellOrder)

AppCameraFrame = _reflection.GeneratedProtocolMessageType(
    "AppCameraFrame",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPCAMERAFRAME,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppCameraFrame)
    },
)
_sym_db.RegisterMessage(AppCameraFrame)

AppTeamChanged = _reflection.GeneratedProtocolMessageType(
    "AppTeamChanged",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPTEAMCHANGED,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppTeamChanged)
    },
)
_sym_db.RegisterMessage(AppTeamChanged)

AppTeamMessage = _reflection.GeneratedProtocolMessageType(
    "AppTeamMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPTEAMMESSAGE,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppTeamMessage)
    },
)
_sym_db.RegisterMessage(AppTeamMessage)

AppEntityChanged = _reflection.GeneratedProtocolMessageType(
    "AppEntityChanged",
    (_message.Message,),
    {
        "DESCRIPTOR": _APPENTITYCHANGED,
        "__module__": "rustplus_proto"
        # @@protoc_insertion_point(class_scope:rustplus.AppEntityChanged)
    },
)
_sym_db.RegisterMessage(AppEntityChanged)

if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._options = None
    _APPENTITYTYPE._serialized_start = 3828
    _APPENTITYTYPE._serialized_end = 3886
    _APPMARKERTYPE._serialized_start = 3889
    _APPMARKERTYPE._serialized_end = 4028
    _APPMESSAGE._serialized_start = 28
    _APPMESSAGE._serialized_end = 124
    _APPREQUEST._serialized_start = 127
    _APPREQUEST._serialized_end = 794
    _APPSENDMESSAGE._serialized_start = 796
    _APPSENDMESSAGE._serialized_end = 829
    _APPSETENTITYVALUE._serialized_start = 831
    _APPSETENTITYVALUE._serialized_end = 865
    _APPCAMERAFRAMEREQUEST._serialized_start = 867
    _APPCAMERAFRAMEREQUEST._serialized_end = 925
    _APPPROMOTETOLEADER._serialized_start = 927
    _APPPROMOTETOLEADER._serialized_end = 964
    _APPRESPONSE._serialized_start = 967
    _APPRESPONSE._serialized_end = 1416
    _APPBROADCAST._serialized_start = 1419
    _APPBROADCAST._serialized_end = 1578
    _APPEMPTY._serialized_start = 1580
    _APPEMPTY._serialized_end = 1590
    _APPSUCCESS._serialized_start = 1592
    _APPSUCCESS._serialized_end = 1604
    _APPERROR._serialized_start = 1606
    _APPERROR._serialized_end = 1631
    _APPINFO._serialized_start = 1634
    _APPINFO._serialized_end = 1827
    _APPTIME._serialized_start = 1829
    _APPTIME._serialized_end = 1930
    _APPMAP._serialized_start = 1933
    _APPMAP._serialized_end = 2126
    _APPMAP_MONUMENT._serialized_start = 2079
    _APPMAP_MONUMENT._serialized_end = 2126
    _APPTEAMINFO._serialized_start = 2129
    _APPTEAMINFO._serialized_end = 2491
    _APPTEAMINFO_MEMBER._serialized_start = 2313
    _APPTEAMINFO_MEMBER._serialized_end = 2447
    _APPTEAMINFO_NOTE._serialized_start = 2449
    _APPTEAMINFO_NOTE._serialized_end = 2491
    _APPTEAMCHAT._serialized_start = 2493
    _APPTEAMCHAT._serialized_end = 2550
    _APPCHATMESSAGE._serialized_start = 2552
    _APPCHATMESSAGE._serialized_end = 2645
    _APPENTITYINFO._serialized_start = 2647
    _APPENTITYINFO._serialized_end = 2746
    _APPENTITYPAYLOAD._serialized_start = 2749
    _APPENTITYPAYLOAD._serialized_end = 2964
    _APPENTITYPAYLOAD_ITEM._serialized_start = 2899
    _APPENTITYPAYLOAD_ITEM._serialized_end = 2964
    _APPFLAG._serialized_start = 2966
    _APPFLAG._serialized_end = 2990
    _APPMAPMARKERS._serialized_start = 2992
    _APPMAPMARKERS._serialized_end = 3045
    _VECTOR4._serialized_start = 3047
    _VECTOR4._serialized_end = 3100
    _APPMARKER._serialized_start = 3103
    _APPMARKER._serialized_end = 3554
    _APPMARKER_SELLORDER._serialized_start = 3391
    _APPMARKER_SELLORDER._serialized_end = 3554
    _APPCAMERAFRAME._serialized_start = 3556
    _APPCAMERAFRAME._serialized_end = 3605
    _APPTEAMCHANGED._serialized_start = 3607
    _APPTEAMCHANGED._serialized_end = 3682
    _APPTEAMMESSAGE._serialized_start = 3684
    _APPTEAMMESSAGE._serialized_end = 3743
    _APPENTITYCHANGED._serialized_start = 3745
    _APPENTITYCHANGED._serialized_end = 3826
# @@protoc_insertion_point(module_scope)



class Error(Exception):
    """Base class for other exceptions"""

    pass


class RateLimitError(Error):
    """Raised When an issue with the ratelimit has occurred"""

    pass


class ServerNotResponsiveError(Error):
    """Raised when the target Server is not online / Unavailable"""

    pass


class CommandsNotEnabledError(Error):
    """Raised when events are not enabled"""

    pass


class ResponseNotReceivedError(Error):
    """Raised when a response has not been received from the server"""

    pass


class PrefixNotDefinedError(Error):
    """Raised when a prefix is not given"""

    pass


class ImageError(Error):
    """Raised when the bytes received are not valid"""

    pass


class ClientNotConnectedError(Error):
    """Raised when you are not connected to the Rust server"""

    pass


class RequestError(Error):
    """Raised when an error is received from the server"""

    pass


class SmartDeviceRegistrationError(Error):
    """Raised when the entity cannot be registered with"""

    pass



from .exceptions import (
    RateLimitError,
    ServerNotResponsiveError,
    CommandsNotEnabledError,
    ResponseNotReceivedError,
    PrefixNotDefinedError,
    ImageError,
    ClientNotConnectedError,
    RequestError,
    SmartDeviceRegistrationError,
)



import time
from threading import Thread

from .conversation import Conversation


class ConversationFactory:
    def __init__(self, api):
        self.api = api
        self.conversations = {}
        self.expires = {}
        self.gc_thread = Thread(target=self.garbage_collect, daemon=True)
        self.gc_thread.start()

    def create_conversation(self, steamid: int) -> Conversation:

        if steamid in self.conversations:
            raise ValueError("Conversation already exists")

        return Conversation(
            api=self.api, target=steamid, register=self._register_conversation
        )

    def _register_conversation(self, steamid, convo: Conversation) -> None:
        self.conversations[steamid] = convo
        self.expires[steamid] = time.time() + 60 * 5

    def has_conversation(self, steamid: int) -> bool:
        return steamid in self.conversations

    def get_conversation(self, steamid: int) -> Conversation:
        return self.conversations[steamid]

    def abort_conversation(self, steamid: int) -> None:
        try:
            del self.conversations[steamid]
            del self.expires[steamid]
        except KeyError:
            pass

    def garbage_collect(self) -> None:
        while True:
            to_remove = []
            for steamid, expire_time in self.expires.items():
                if expire_time < time.time():
                    to_remove.append(steamid)

            for steamid in to_remove:
                self.abort_conversation(steamid)

            time.sleep(5)



import asyncio
from typing import List
from .conversation_prompt import ConversationPrompt


class Conversation:
    def __init__(
        self,
        api,
        target: int = None,
        prompts: List[ConversationPrompt] = None,
        register=None,
    ) -> None:

        if target is None:
            raise ValueError("target must be specified")
        self._target = target

        if prompts is None:
            self._prompts = []
        else:
            self._prompts = prompts

        self._answers = []
        self._seq = 0
        self._api = api
        self._loop = None
        self._register = register

    def add_prompt(self, prompt: ConversationPrompt) -> None:
        super(type(prompt), prompt).__init__(self)
        self._prompts.append(prompt)

    def add_all_prompts(self, prompts: List[ConversationPrompt]) -> None:
        for prompt in prompts:
            super(ConversationPrompt, prompt).__init__(self)
            self.add_prompt(prompt)

    def has_next(self) -> bool:
        return self._seq + 1 < len(self._prompts)

    def get_current_prompt(self) -> ConversationPrompt:
        return self._prompts[self._seq]

    def increment_prompt(self) -> None:
        self._seq += 1

    async def send_prompt(self, message: str) -> None:
        if self._api.remote.ws is not None:
            self._api.remote.ws.outgoing_conversation_messages.append(message)
        await self._api.send_team_message(message)

    async def start(self) -> None:
        self._register(self._target, self)
        self._loop = asyncio.get_event_loop_policy().get_event_loop()
        await self.send_prompt(await self._prompts[0].prompt())

    def run_coro(self, coro, args):
        return asyncio.run_coroutine_threadsafe(coro(*args), self._loop).result()

    def get_answers(self) -> List[str]:
        return self._answers



from .rust_time import RustTime
from .rust_info import RustInfo
from .rust_map import RustMap
from .rust_marker import RustMarker
from .rust_chat_message import RustChatMessage
from .rust_team_info import RustTeamInfo, RustTeamMember, RustTeamNote
from .rust_entity_info import RustEntityInfo
from .rust_contents import RustContents
from .rust_item import RustItem
from .event import EntityEvent, TeamEvent, ChatEvent



from typing import List


class RustTeamMember:
    def __init__(self, data) -> None:
        self._steam_id: int = data.steamId
        self._name: str = data.name
        self._x: float = data.x
        self._y: float = data.y
        self._is_online: bool = data.isOnline
        self._spawn_time: int = data.spawnTime
        self._is_alive: bool = data.isAlive
        self._death_time: int = data.deathTime

    @property
    def steam_id(self) -> int:
        return self._steam_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def x(self) -> float:
        return self._x

    @property
    def y(self) -> float:
        return self._y

    @property
    def is_online(self) -> bool:
        return self._is_online

    @property
    def spawn_time(self) -> int:
        return self._spawn_time

    @property
    def is_alive(self) -> bool:
        return self._is_alive

    @property
    def death_time(self) -> int:
        return self._death_time

    def __str__(self) -> str:
        return "RustTeamMember[steam_id={}, name={}, x={}, y={}, is_online={}, spawn_time={}, is_alive={}, death_time={}]".format(
            self._steam_id,
            self._name,
            self._x,
            self._y,
            self._is_online,
            self._spawn_time,
            self._is_alive,
            self._death_time,
        )


class RustTeamNote:
    def __init__(self, data) -> None:
        self._type: int = data.type
        self._x: float = data.x
        self._y: float = data.y

    @property
    def type(self) -> int:
        return self._type

    @property
    def x(self) -> float:
        return self._x

    @property
    def y(self) -> float:
        return self._y

    def __str__(self) -> str:
        return "RustTeamNote[type={}, x={}, y={}]".format(self._type, self._x, self._y)


class RustTeamInfo:
    def __init__(self, data) -> None:
        self._leader_steam_id: int = data.leaderSteamId
        self._members = [RustTeamMember(member) for member in data.members]
        self._map_notes = [RustTeamNote(note) for note in data.mapNotes]
        self._leader_map_notes = [RustTeamNote(note) for note in data.leaderMapNotes]

    @property
    def leader_steam_id(self) -> int:
        return self._leader_steam_id

    @property
    def members(self) -> List[RustTeamMember]:
        return self._members

    @property
    def map_notes(self) -> List[RustTeamNote]:
        return self._map_notes

    @property
    def leader_map_notes(self) -> List[RustTeamNote]:
        return self._leader_map_notes

    def __str__(self) -> str:
        return "RustTeamInfo[leader_steam_id={}, members={}, map_notes={}, leader_map_notes={}]".format(
            self._leader_steam_id,
            self._members,
            self._map_notes,
            self._leader_map_notes,
        )



class RustInfo:
    def __init__(self, data) -> None:
        self._url: str = data.url
        self._name: str = data.name
        self._map: str = data.map
        self._size: int = data.mapSize
        self._players: int = data.players
        self._max_players: int = data.maxPlayers
        self._queued_players: int = data.queuedPlayers
        self._seed: int = data.seed
        self._wipe_time: int = data.wipeTime

    @property
    def url(self) -> str:
        return self._url

    @property
    def name(self) -> str:
        return self._name

    @property
    def map(self) -> str:
        return self._map

    @property
    def size(self) -> int:
        return self._size

    @property
    def players(self) -> int:
        return self._players

    @property
    def max_players(self) -> int:
        return self._max_players

    @property
    def queued_players(self) -> int:
        return self._queued_players

    @property
    def seed(self) -> int:
        return self._seed

    @property
    def wipe_time(self) -> int:
        return self._wipe_time

    def __str__(self) -> str:
        return (
            "RustInfo[url={}, name={}, map={}, size={}, players={}, max_players={}, queued_players={}, seed={}, "
            "wipe_time={}]".format(
                self._url,
                self._name,
                self._map,
                self._size,
                self._players,
                self._max_players,
                self._queued_players,
                self._seed,
                self._wipe_time,
            )
        )



import asyncio
import logging
import time

from .event_handler import EventHandler
from .rustplus_proto import *
from .rustws import RustWebsocket, CONNECTED, PENDING_CONNECTION
from .token_bucket import RateLimiter
from .expo_bundle_handler import MagicValueGrabber
from ...conversation import ConversationFactory
from ...commands import CommandHandler
from ...exceptions import (
    ClientNotConnectedError,
    ResponseNotReceivedError,
    RequestError,
)
from ...utils import RegisteredListener


class RustRemote:
    def __init__(
        self,
        ip,
        port,
        command_options,
        ratelimit_limit,
        ratelimit_refill,
        websocket_length=600,
        use_proxy: bool = False,
        api=None,
        loop=None,
    ) -> None:

        self.ip = ip
        self.port = port
        self.command_options = command_options
        self.ratelimit_limit = ratelimit_limit
        self.ratelimit_refill = ratelimit_refill
        self.use_proxy = use_proxy
        self.ratelimiter = RateLimiter(
            ratelimit_limit, ratelimit_limit, 1, ratelimit_refill
        )
        self.ws = None
        self.websocket_length = websocket_length
        self.responses = {}
        self.ignored_responses = []
        self.pending_for_response = {}
        self.sent_requests = []

        if command_options is None:
            self.use_commands = False
        else:
            self.use_commands = True
            self.command_options = command_options
            self.command_handler = CommandHandler(self.command_options)

        self.event_handler = EventHandler()

        self.magic_value = MagicValueGrabber.get_magic_value()
        self.conversation_factory = ConversationFactory(api)
        self.loop = loop

    def connect(self, retries, delay) -> None:

        self.ws = RustWebsocket(
            ip=self.ip,
            port=self.port,
            remote=self,
            use_proxy=self.use_proxy,
            magic_value=self.magic_value,
        )
        self.ws.connect(retries=retries, delay=delay)

    def close(self) -> None:

        if self.ws is not None:
            self.ws.close()
            del self.ws
            self.ws = None

    def is_pending(self) -> bool:
        if self.ws is not None:
            return self.ws.connection_status == PENDING_CONNECTION
        return False

    def is_open(self) -> bool:
        if self.ws is not None:
            return self.ws.connection_status == CONNECTED
        return False

    async def send_message(self, request: AppRequest) -> None:

        await self.ws.send_message(request)

    async def get_response(
        self, seq: int, app_request: AppRequest, error_check: bool = True
    ) -> AppMessage:
        """
        Returns a given response from the server.
        """

        attempts = 0

        while seq in self.pending_for_response and seq not in self.responses:

            if seq in self.sent_requests:

                if attempts <= 40:

                    attempts += 1
                    await asyncio.sleep(0.1)

                else:

                    await self.send_message(app_request)
                    await asyncio.sleep(0.1)
                    attempts = 0

            if attempts <= 10:
                await asyncio.sleep(0.1)
                attempts += 1

            else:
                await self.send_message(app_request)
                await asyncio.sleep(1)
                attempts = 0

        if seq not in self.responses:
            raise ResponseNotReceivedError("Not Received")

        response = self.responses.pop(seq)

        if response.response.error.error == "rate_limit":
            logging.getLogger("rustplus.py").warning(
                "[Rustplus.py] RateLimit Exception Occurred. Retrying after bucket is full"
            )

            # Fully Refill the bucket

            self.ratelimiter.last_consumed = time.time()
            self.ratelimiter.bucket.current = 0

            while self.ratelimiter.bucket.current < self.ratelimiter.bucket.max:
                await asyncio.sleep(1)
                self.ratelimiter.bucket.refresh()

            # Reattempt the sending with a full bucket
            cost = self.ws.get_proto_cost(app_request)

            while True:

                if self.ratelimiter.can_consume(cost):
                    self.ratelimiter.consume(cost)
                    break

                await asyncio.sleep(self.ratelimiter.get_estimated_delay_time(cost))

            await self.send_message(app_request)
            response = await self.get_response(seq, app_request)

        elif self.ws.error_present(response.response.error.error) and error_check:
            raise RequestError(response.response.error.error)

        return response

    def _sock(self, retries) -> RustWebsocket:

        if self.ws is None:
            raise ClientNotConnectedError("No Current Websocket Connection")

        while self.is_pending():
            time.sleep(1)

        if time.time() - self.ws.connected_time >= self.websocket_length:
            self.close()
            self.connect(retries=retries, delay=20)

        return self.ws

    def remove_listener(self, listener: RegisteredListener) -> bool:

        if self.event_handler.has_event(listener):
            self.event_handler.remove_event(listener)
            return True

        elif self.use_commands:
            if self.command_handler.has_command(listener):
                self.command_handler.remove_command(listener)
                return True

        return False



import asyncio
from asyncio.futures import Future
from datetime import datetime

from . import Command, CommandTime
from ..api.structures import RustChatMessage
from ..commands.command_options import CommandOptions
from ..commands.command_data import CommandData
from ..utils import RegisteredListener


class CommandHandler:
    def __init__(self, command_options: CommandOptions) -> None:
        self.command_options = command_options
        self.commands = {}

    def register_command(self, data: CommandData) -> None:

        if not asyncio.iscoroutinefunction(data.coro):
            raise TypeError("The event registered must be a coroutine")

        self.commands[data.coro.__name__] = data

    @staticmethod
    def _schedule_event(loop, coro, arg) -> None:
        def callback(inner_future: Future):
            inner_future.result()

        future: Future = asyncio.run_coroutine_threadsafe(coro(arg), loop)
        future.add_done_callback(callback)

    def run_command(self, message: RustChatMessage, prefix) -> None:

        if prefix == self.command_options.prefix:
            command = message.message.split(" ")[0][len(prefix) :]
        else:
            command = prefix

        if command in self.commands:
            data = self.commands[command]

            self._schedule_event(
                data.loop,
                data.coro,
                Command(
                    message.name,
                    message.steam_id,
                    CommandTime(datetime.utcfromtimestamp(message.time), message.time),
                    command,
                    message.message.split(" ")[1:],
                ),
            )
        else:
            for command_name, data in self.commands.items():
                # Loop through all the commands and see if the command is in the data aliases list
                # or if it matches the callable function

                if command in data.aliases or data.callable_func(command):
                    self._schedule_event(
                        data.loop,
                        data.coro,
                        Command(
                            message.name,
                            message.steam_id,
                            CommandTime(
                                datetime.utcfromtimestamp(message.time), message.time
                            ),
                            command,
                            message.message.split(" ")[1:],
                        ),
                    )
                    break

    def has_command(self, listener: RegisteredListener) -> bool:
        return listener.listener_id in self.commands

    def remove_command(self, listener: RegisteredListener) -> None:
        try:
            del self.commands[listener.listener_id]
        except KeyError:
            pass



from setuptools import setup, find_packages

version = ""
with open("./rustplus/__init__.py") as input_file:
    for line in input_file.readlines():
        if line.startswith("__version__"):
            version = line.strip("__version__ =").strip().strip('"')
            break

with open("README.md", errors='ignore') as input_file:
    readme = input_file.read()

setup(
    name='rustplus',
    author='olijeffers0n',
    author_email='pleaseUseMyDiscord@Please.com',
    url='https://github.com/olijeffers0n/rustplus',
    project_urls={
        "Issue tracker": "https://github.com/olijeffers0n/rustplus/issues",
    },
    version=version,
    include_package_data=True,
    packages=find_packages(include=['rustplus', 'rustplus.*']),
    license='MIT',
    description='A python wrapper for the Rust Plus API',
    long_description=readme,
    long_description_content_type='text/markdown',
    install_requires=[
        "websocket_client",
        "Pillow",
        "protobuf>=3.20.0",
        "asyncio",
        "push_receiver",
        "http-ece",
        "requests"
    ],
    python_requires='>=3.7.0',
)



class RustTime:
    def __init__(self, day_length, sunrise, sunset, time, raw_time) -> None:
        self._day_length: float = day_length
        self._sunrise: str = sunrise
        self._sunset: str = sunset
        self._time: str = time
        self._raw_time: float = raw_time

    @property
    def day_length(self) -> float:
        return self._day_length

    @property
    def sunrise(self) -> str:
        return self._sunrise

    @property
    def sunset(self) -> str:
        return self._sunset

    @property
    def time(self) -> str:
        return self._time

    @property
    def raw_time(self) -> float:
        return self._raw_time

    def __str__(self) -> str:
        return "RustTime[day_length={}, sunrise={}, sunset={}, time={}, raw_time={}]".format(
            self._day_length, self._sunrise, self._sunset, self._time, self._raw_time
        )



import asyncio
import logging
import time
from datetime import datetime
from threading import Thread
from typing import Optional
import websocket

from .rustplus_proto import AppMessage, AppRequest
from ..structures import RustChatMessage
from ...exceptions import ClientNotConnectedError
from ...conversation import Conversation

CONNECTED = 1
PENDING_CONNECTION = 2
CLOSING = 4
CLOSED = 3


class RustWebsocket(websocket.WebSocket):
    def __init__(self, ip, port, remote, use_proxy, magic_value):

        self.ip = ip
        self.port = port
        self.thread: Thread = None
        self.connection_status = CLOSED
        self.use_proxy = use_proxy
        self.remote = remote
        self.logger = logging.getLogger("rustplus.py")
        self.connected_time = time.time()
        self.magic_value = magic_value
        self.outgoing_conversation_messages = []

        super().__init__(enable_multithread=True)

    def connect(
        self, retries=float("inf"), ignore_open_value: bool = False, delay: int = 20
    ) -> None:

        if (
            not self.connection_status == CONNECTED or ignore_open_value
        ) and not self.remote.is_pending():

            attempts = 0

            while True:

                if attempts >= retries:
                    raise ConnectionAbortedError("Reached Retry Limit")

                self.connection_status = PENDING_CONNECTION

                try:
                    address = (
                        f"wss://companion-rust.facepunch.com/game/{self.ip}/{self.port}?v={str(self.magic_value)}"
                        if self.use_proxy
                        else f"ws://{self.ip}:{self.port}?v={str(self.magic_value)}"
                    )
                    super().connect(address)
                    self.connected_time = time.time()
                    break
                except Exception:
                    self.logger.warning(
                        f"{datetime.now().strftime('%d/%m/%Y %H:%M:%S')} "
                        f"[RustPlus.py] Cannot Connect to server. Retrying in {str(delay)} seconds"
                    )
                    attempts += 1
                    time.sleep(delay)

            self.connection_status = CONNECTED

        if not ignore_open_value:

            self.thread = Thread(
                target=self.run, name="[RustPlus.py] WebsocketThread", daemon=True
            )
            self.thread.start()

    def close(self) -> None:

        self.connection_status = CLOSING
        super().close()
        self.connection_status = CLOSED

    async def send_message(self, message: AppRequest) -> None:
        """
        Send the Protobuf to the server
        """

        if self.connection_status == CLOSED:
            raise ClientNotConnectedError("Not Connected")

        try:
            self.send_binary(message.SerializeToString())
            self.remote.pending_for_response[message.seq] = message
        except Exception:
            while self.remote.is_pending():
                await asyncio.sleep(0.5)
            return await self.remote.send_message(message)

    def run(self) -> None:

        while self.connection_status == CONNECTED:
            try:
                data = self.recv()

                self.remote.event_handler.run_proto_event(data)

                app_message = AppMessage()
                app_message.ParseFromString(data)

            except Exception:
                if self.connection_status == CONNECTED:
                    self.logger.warning(
                        f"{datetime.now().strftime('%d/%m/%Y %H:%M:%S')} [RustPlus.py] Connection interrupted, Retrying"
                    )
                    self.connect(ignore_open_value=True)
                    continue
                return

            try:
                del self.remote.pending_for_response[app_message.response.seq]
            except KeyError:
                pass

            self.handle_message(app_message)

    def handle_message(self, app_message: AppMessage) -> None:

        if app_message.response.seq in self.remote.ignored_responses:
            self.remote.ignored_responses.remove(app_message.response.seq)
            return

        prefix = self.get_prefix(str(app_message.broadcast.teamMessage.message.message))

        if prefix is not None:
            # This means it is a command

            message = RustChatMessage(app_message.broadcast.teamMessage.message)

            self.remote.command_handler.run_command(message, prefix)

        if self.is_entity_broadcast(app_message):
            # This means that an entity has changed state

            self.remote.event_handler.run_entity_event(
                app_message.broadcast.entityChanged.entityId, app_message
            )

        elif self.is_team_broadcast(app_message):
            # This means that the team of the current player has changed
            self.remote.event_handler.run_team_event(app_message)

        elif self.is_message(app_message):
            # This means that a message has been sent to the team chat

            steam_id = int(app_message.broadcast.teamMessage.message.steamId)
            message = str(app_message.broadcast.teamMessage.message.message)

            if self.remote.conversation_factory.has_conversation(steam_id):
                if message not in self.outgoing_conversation_messages:
                    conversation: Conversation = (
                        self.remote.conversation_factory.get_conversation(steam_id)
                    )

                    conversation.get_answers().append(message)
                    conversation.run_coro(
                        conversation.get_current_prompt().on_response, args=[message]
                    )

                    if conversation.has_next():
                        conversation.increment_prompt()
                        prompt = conversation.get_current_prompt()
                        prompt_string = conversation.run_coro(prompt.prompt, args=[])
                        conversation.run_coro(
                            conversation.send_prompt, args=[prompt_string]
                        )
                    else:
                        prompt = conversation.get_current_prompt()
                        prompt_string = conversation.run_coro(prompt.on_finish, args=[])
                        if prompt_string != "":
                            conversation.run_coro(
                                conversation.send_prompt, args=[prompt_string]
                            )
                        self.remote.conversation_factory.abort_conversation(steam_id)
                else:
                    self.outgoing_conversation_messages.remove(message)

            self.remote.event_handler.run_chat_event(app_message)

        else:
            # This means that it wasn't sent by the server and is a message from the server in response to an action

            self.remote.responses[app_message.response.seq] = app_message

    def get_prefix(self, message: str) -> Optional[str]:
        if self.remote.use_commands:
            if message.startswith(self.remote.command_options.prefix):
                return self.remote.command_options.prefix
        else:
            return None

        for overrule in self.remote.command_options.overruling_commands:
            if message.startswith(overrule):
                return overrule

        return None

    @staticmethod
    def is_message(app_message) -> bool:
        return str(app_message.broadcast.teamMessage.message.message) != ""

    @staticmethod
    def is_entity_broadcast(app_message) -> bool:
        return str(app_message.broadcast.entityChanged) != ""

    @staticmethod
    def is_team_broadcast(app_message) -> bool:
        return str(app_message.broadcast.teamChanged) != ""

    async def _retry_failed_request(self, app_request: AppRequest):
        """
        Resends an AppRequest to the server if it has failed
        """
        await self.send_message(app_request)

    @staticmethod
    def get_proto_cost(app_request) -> int:
        """
        Gets the cost of an AppRequest
        """
        costs = {
            "getTime": 1,
            "sendTeamMessage": 2,
            "getInfo": 1,
            "getTeamChat": 1,
            "getTeamInfo": 1,
            "getMapMarkers": 1,
            "getMap": 5,
            "setEntityValue": 1,
            "getEntityInfo": 1,
            "promoteToLeader": 1,
        }
        for request, cost in costs.items():
            if app_request.HasField(request):
                return cost

        raise ValueError()

    @staticmethod
    def error_present(message) -> bool:
        """
        Checks message for error
        """
        return message != ""



class RegisteredListener:
    def __init__(self, listener_id: str, data) -> None:
        self.listener_id = listener_id
        self.data = data

    def get_coro(self):
        return self.data[0]

    def __eq__(self, other):
        if isinstance(other, RegisteredListener):
            return self.listener_id == other.listener_id and self.data == other.data
        return False



from typing import Callable, List


class CommandData:
    def __init__(self, coro, loop, aliases, callable_func) -> None:
        self.coro = coro
        self.loop = loop
        self._aliases = aliases
        self._callable_func = callable_func

    @property
    def aliases(self) -> List[str]:
        if self._aliases is None:
            return []

        return self._aliases

    @property
    def callable_func(self) -> Callable:
        if self._callable_func is None:
            return lambda x: False

        return self._callable_func



from datetime import timedelta
from typing import List

from .rust_item import RustItem


class RustContents:
    def __init__(self, protection_time, has_protection, contents) -> None:
        self._protection_time: timedelta = protection_time
        self._has_protection: bool = has_protection
        self._contents: List[RustItem] = contents

    @property
    def protection_time(self) -> timedelta:
        return self._protection_time

    @property
    def has_protection(self) -> bool:
        return self._has_protection

    @property
    def contents(self) -> List[RustItem]:
        return self._contents

    def __str__(self) -> str:
        return (
            "RustContents[protection_time={}, has_protection={}, contents={}]".format(
                self._protection_time, self._has_protection, self.contents
            )
        )



class RustChatMessage:
    def __init__(self, data):
        self._steam_id: int = data.steamId
        self._name: str = data.name
        self._message: str = data.message
        self._colour: str = data.color
        self._time: int = data.time

    @property
    def steam_id(self) -> int:
        return self._steam_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def message(self) -> str:
        return self._message

    @property
    def colour(self) -> str:
        return self._colour

    @property
    def time(self) -> int:
        return self._time

    def __repr__(self):
        return "RustChatMessage[steam_id={}, sender_name={}, message={}, colour={}, time={}]".format(
            self._steam_id, self._name, self._message, self._colour, self._time
        )
